unit ExcelFormulaToken;

{
Copyright: Algorithm (c) E. W. Bachtal, Inc

Permission is hereby granted, free of charge, to any person obtaining a copy of this software
and associated documentation files (the "Software"), to deal in the Software without restriction,
including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all copies or substantial
  portions of the Software.

The software is provided "as is", without warranty of any kind, express or implied, including but not
limited to the warranties of merchantability, fitness for a particular purpose and noninfringement. In
no event shall the authors or copyright holders be liable for any claim, damages or other liability,
whether in an action of contract, tort or otherwise, arising from, out of or in connection with the
software or the use or other dealings in the software.

This is a Delphi XE port of ewbi's Excel formula parser.  ( hadv.de )

Original Javascript post is: http://ewbi.blogs.com/develops/2004/12/excel_formula_p.html

Other ports are:
  C#:      http://ewbi.blogs.com/develops/2007/03/excel_formula_p.html
  Python:  http://www.ewbi.com/ewbi.develop/samples/jsport_nonEAT.html
  C++:     https://github.com/lishen2/Excel_formula_parser_cpp

Versions:
  1.0:    2011/06/13 - Orginal Delphi XE port by hadv.de
}

interface

uses
 Generics.Collections;

type

TExcelFormulaTokenType = (
  enNoop,
  enOperand,
  enFunction,
  enSubexpression,
  enArgument,
  enOperatorPrefix,
  enOperatorInfix,
  enOperatorPostfix,
  enWhitespace,
  enUnknown
);

TExcelFormulaTokenSubtype = (
  enNothing,
  enStart,
  enStop,
  enText,
  enNumber,
  enLogical,
  enError,
  enRange,
  enMath,
  enConcatenation,
  enIntersection,
  enUnion
);


TExcelFormulaToken = class
 private
   FValue:   String;
   FType:    TExcelFormulaTokenType;
   FSubType: TExcelFormulaTokenSubtype;
 public
   constructor Create(TokValue: String; TokType: TExcelFormulaTokenType); overload;
   constructor Create(TokValue: String; TokType: TExcelFormulaTokenType; TokSubType: TExcelFormulaTokenSubtype); overload;
   function    Clone: TExcelFormulaToken;

   function  GetTokenTypeString: String;
   function  GetTokenSubTypeString: String;

   property  TokValue:   String read FValue write FValue;
   property  TokType:    TExcelFormulaTokenType read FType write FType;
   property  TokSubType: TExcelFormulaTokenSubtype read FSubType write FSubType;
end;

implementation

constructor TExcelFormulaToken.Create(TokValue: String; TokType: TExcelFormulaTokenType);
begin
 self.FValue := TokValue;
 self.FType  := TokType;
 self.FSubType := TExcelFormulaTokenSubtype.enNothing;
end;

constructor TExcelFormulaToken.Create(TokValue: String; TokType: TExcelFormulaTokenType; TokSubType: TExcelFormulaTokenSubtype);
begin
 self.FValue := TokValue;
 self.FType  := TokType;
 self.FSubType := TokSubType;
end;

function TExcelFormulaToken.Clone: TExcelFormulaToken;
begin
 Result := TExcelFormulaToken.Create(self.FValue,self.FType, self.FSubType);
end;

function TExcelFormulaToken.GetTokenTypeString: String;
begin
 case self.FType of
   enNoop:          Result:= 'Noop';
   enOperand:       Result:= 'Operand';
   enFunction:      Result:= 'Function';
   enSubexpression: Result:= 'Subexpression';
   enArgument: Result:= 'Argument';
   enOperatorPrefix: Result:= 'OperatorPrefix';
   enOperatorInfix:  Result:= 'OperatorInfix';
   enOperatorPostfix: Result:= 'OperatorPostfix';
   enWhitespace:     Result:= 'Whitespace';
   enUnknown:        Result:= 'Unknown';
 else
   Result:= 'Unknown';
 end;
end;

function TExcelFormulaToken.GetTokenSubTypeString: String;
begin
 case self.FSubType of
   enNothing:       Result:= '';
   enStart:         Result:= 'Start';
   enStop:          Result:= 'Stop';
   enText:          Result:= 'Text';
   enNumber:        Result:= 'Number';
   enLogical:       Result:= 'Logical';
   enError:         Result:= 'Error';
   enRange:         Result:= 'Range';
   enMath:          Result:= 'Math';
   enConcatenation: Result:= 'Concatenation';
   enIntersection:  Result:= 'Intersection';
   enUnion:         Result:= 'Union';
 else
   Result:= '???';
 end;
end;

end.
